/*
  more_averages.cc

  Written by Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <vector>
#include <functional>
#if __cplusplus < 201103L
#include <boost/function.hpp>
#endif
#include <cstdlib>      // random(), etc
#include <cassert>
#include "averages.hpp"
#include "averages_online.hpp"

int main()
{
  const double maximum_permissible_error(1e-13);
  const char *result_names[] = {
    "exp-log mean",
  };
  std::vector<double> storage(10000);
  generic_average_online<double> *online_averages[] = {
    new arbitrary_generalized_average_online<double>(exp, log),
  };
  std::string tabs_for_result_names[sizeof(result_names)/sizeof(result_names[0])];
  {
    size_t l = std::string(result_names[0]).size();
    for (size_t i=1; i<sizeof(result_names)/sizeof(result_names[0]); ++i)
      l = std::max(l, std::string(result_names[i]).size());
    for (size_t i=0; i<sizeof(result_names)/sizeof(result_names[0]); ++i) {
      tabs_for_result_names[i] = "\t";
      for (size_t j=0; j<(l+1)/8 - (std::string(result_names[i]).size()+1)/8; ++j)
	tabs_for_result_names[i] += "\t";
    }
  }
  srandomdev();
  for (size_t i=0; i<storage.size(); ++i) {
    double v((double)random()/((1L<<31)-1));
    storage[i] = v;
    for (size_t j=0; j<sizeof(online_averages)/sizeof(online_averages[0]); ++j)
      online_averages[j]->average(v);
  }
  double averages[] = {
    generalized_average(storage.begin(), storage.end(), exp, log),
  };
  for (size_t j=0; j<sizeof(online_averages)/sizeof(online_averages[0]); ++j) {
    double error(sqrt(pow(online_averages[j]->get_value() - averages[j], 2)));
    std::cout << result_names[j] << ":" << tabs_for_result_names[j] << online_averages[j]->get_value() << "\t" << averages[j] << "\t" << error << std::endl;
    assert(error < maximum_permissible_error);
  }
  std::vector<double> moving_storage(storage.size());
  generic_average_online<double> *online_moving_averages[] = {
    new moving_arbitrary_generalized_average_online<double>(storage.size(), exp, log),
  };
  for (size_t i=0; i<storage.size()*10; ++i) {
    double v((double)random()/((1L<<31)-1));
    moving_storage[i % storage.size()] = v;
    for (size_t j=0; j<sizeof(online_moving_averages)/sizeof(online_moving_averages[0]); ++j)
      online_moving_averages[j]->average(v);
    if (i < storage.size()) continue;
    double moving_averages[] = {
      generalized_average(moving_storage.begin(), moving_storage.end(), exp, log),
    };
    for (size_t j=0; j<sizeof(online_moving_averages)/sizeof(online_moving_averages[0]); ++j) {
      double error(sqrt(pow(online_moving_averages[j]->get_value() - moving_averages[j], 2)));
      if (i+1 == storage.size()*10)
        std::cout << result_names[j] << ":" << tabs_for_result_names[j] << online_moving_averages[j]->get_value() << "\t" << moving_averages[j] << "\t" << error << std::endl;
      assert(error < maximum_permissible_error);
    }
  }
  return 0;
}
